﻿;Copyright (C) 2004-2012 John T. Haller of PortableApps.com

;Website: http://portableapps.com/KeePassPortable

;This software is OSI Certified Open Source Software.
;OSI Certified is a certification mark of the Open Source Initiative.

;This program is free software; you can redistribute it and/or
;modify it under the terms of the GNU General Public License
;as published by the Free Software Foundation; either version 2
;of the License, or (at your option) any later version.

;This program is distributed in the hope that it will be useful,
;but WITHOUT ANY WARRANTY; without even the implied warranty of
;MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
;GNU General Public License for more details.

;You should have received a copy of the GNU General Public License
;along with this program; if not, write to the Free Software
;Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.

!define PORTABLEAPPNAME "KeePass Portable"
!define APPNAME "KeePass"
!define NAME "KeePassPortable"
!define VER "1.7.0.0"
!define WEBSITE "PortableApps.com/KeePassPortable"
!define DEFAULTEXE "keepass.exe"
!define DEFAULTAPPDIR "keepass"
!define LAUNCHERLANGUAGE "English"

;=== Program Details
Name "${PORTABLEAPPNAME}"
OutFile "..\..\${NAME}.exe"
Caption "${PORTABLEAPPNAME} | PortableApps.com"
VIProductVersion "${VER}"
VIAddVersionKey ProductName "${PORTABLEAPPNAME}"
VIAddVersionKey Comments "Allows ${APPNAME} to be run from a removable drive.  For additional details, visit ${WEBSITE}"
VIAddVersionKey CompanyName "PortableApps.com"
VIAddVersionKey LegalCopyright "John T. Haller"
VIAddVersionKey FileDescription "${PORTABLEAPPNAME}"
VIAddVersionKey FileVersion "${VER}"
VIAddVersionKey ProductVersion "${VER}"
VIAddVersionKey InternalName "${PORTABLEAPPNAME}"
VIAddVersionKey LegalTrademarks "PortableApps.com is a Trademark of Rare Ideas, LLC."
VIAddVersionKey OriginalFilename "${NAME}.exe"
;VIAddVersionKey PrivateBuild ""
;VIAddVersionKey SpecialBuild ""

;=== Runtime Switches
CRCCheck On
WindowIcon Off
SilentInstall Silent
AutoCloseWindow True
RequestExecutionLevel user
XPStyle on

; Best Compression
SetCompress Auto
SetCompressor /SOLID lzma
SetCompressorDictSize 32
SetDatablockOptimize On

;=== Include
;Standard
!include FileFunc.nsh
!insertmacro GetParameters
!insertmacro GetRoot

;(Custom)
!include ReadINIStrWithDefault.nsh

;=== Program Icon
Icon "..\..\App\AppInfo\appicon.ico"

;=== Icon & Stye ===
;!define MUI_ICON "..\..\App\AppInfo\appicon.ico"

;=== Languages
;!insertmacro MUI_LANGUAGE "${LAUNCHERLANGUAGE}"
LoadLanguageFile "${NSISDIR}\Contrib\Language files\${LAUNCHERLANGUAGE}.nlf"
!include PortableApps.comLauncherLANG_${LAUNCHERLANGUAGE}.nsh

Var PROGRAMDIRECTORY
Var SETTINGSDIRECTORY
Var ADDITIONALPARAMETERS
Var EXECSTRING
Var PROGRAMEXECUTABLE
Var SECONDARYLAUNCH
Var DISABLESPLASHSCREEN
Var MISSINGFILEORPATH
Var APPLANGUAGE

Section "Main"
	;=== Check if already running
	System::Call 'kernel32::CreateMutex(i 0, i 0, t "${NAME}") i .r1 ?e'
	Pop $0
	StrCmp $0 0 CheckINI
		StrCpy $SECONDARYLAUNCH "true"

	CheckINI:
		;=== Find the INI file, if there is one
		IfFileExists "$EXEDIR\${NAME}.ini" "" NoINI
		;=== Read the parameters from the INI file
		${ReadINIStrWithDefault} $0 "$EXEDIR\${NAME}.ini" "${NAME}" "${APPNAME}Directory" "App\${DEFAULTAPPDIR}"
		StrCpy "$PROGRAMDIRECTORY" "$EXEDIR\$0"
		${ReadINIStrWithDefault} $0 "$EXEDIR\${NAME}.ini" "${NAME}" "SettingsDirectory" "Data\settings"
		StrCpy "$SETTINGSDIRECTORY" "$EXEDIR\$0"
		${ReadINIStrWithDefault} $ADDITIONALPARAMETERS "$EXEDIR\${NAME}.ini" "${NAME}" "AdditionalParameters" ""
		${ReadINIStrWithDefault} $DISABLESPLASHSCREEN "$EXEDIR\${NAME}.ini" "${NAME}" "DisableSplashScreen" "false"
		StrCpy "$PROGRAMEXECUTABLE" "${DEFAULTEXE}"
		Goto EndINI

	NoINI:
		;=== No INI file, so we'll use the defaults
		StrCpy "$ADDITIONALPARAMETERS" ""
		StrCpy "$PROGRAMEXECUTABLE" "${DEFAULTEXE}"

		IfFileExists "$EXEDIR\App\${DEFAULTAPPDIR}\${DEFAULTEXE}" "" NoProgramEXE
			StrCpy "$PROGRAMDIRECTORY" "$EXEDIR\App\${DEFAULTAPPDIR}"
			StrCpy "$SETTINGSDIRECTORY" "$EXEDIR\Data\settings"
			GoTo EndINI

	EndINI:
		IfFileExists "$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE" FoundProgramEXE

	NoProgramEXE:
		;=== Program executable not where expected
		StrCpy $MISSINGFILEORPATH $PROGRAMEXECUTABLE
		MessageBox MB_OK|MB_ICONEXCLAMATION `$(LauncherFileNotFound)`
		Abort
		
	FoundProgramEXE:
		;=== Check if running
		StrCmp $SECONDARYLAUNCH "true" GetPassedParameters
		FindProcDLL::FindProc "${DEFAULTEXE}"
		StrCmp $R0 "1" WarnAnotherInstance DisplaySplash

	WarnAnotherInstance:
		MessageBox MB_OK|MB_ICONINFORMATION `$(LauncherAlreadyRunning)`
		Abort
	
	DisplaySplash:
		StrCmp $DISABLESPLASHSCREEN "true" GetPassedParameters
			;=== Show the splash screen while processing registry entries
			InitPluginsDir
			File /oname=$PLUGINSDIR\splash.jpg "${NAME}.jpg"
			newadvsplash::show /NOUNLOAD 1200 0 0 -1 /L $PLUGINSDIR\splash.jpg
	
	GetPassedParameters:
		;=== Get any passed parameters
		${GetParameters} $0
		StrCmp "'$0'" "''" "" LaunchProgramParameters

		;=== No parameters
		StrCpy $EXECSTRING `"$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE"`
		Goto AdditionalParameters

	LaunchProgramParameters:
		StrCpy $EXECSTRING `"$PROGRAMDIRECTORY\$PROGRAMEXECUTABLE" $0`

	AdditionalParameters:
		StrCmp $ADDITIONALPARAMETERS "" SettingsDirectory

		;=== Additional Parameters
		StrCpy $EXECSTRING `$EXECSTRING $ADDITIONALPARAMETERS`
	
	SettingsDirectory:
		;=== Set the settings directory if we have a path
		IfFileExists "$SETTINGSDIRECTORY\KeePass.ini" CheckForSettings
			CreateDirectory $SETTINGSDIRECTORY
	
	CheckForSettings:
		StrCmp $SECONDARYLAUNCH "true" LaunchAndExit
		IfFileExists "$PROGRAMDIRECTORY\KeePass.ini" ReconstructLastDir
		IfFileExists "$SETTINGSDIRECTORY\KeePass.ini" MoveSettings
		IfFileExists "$EXEDIR\App\DefaultData\KeePass.ini" "" LaunchNow
			CopyFiles /SILENT "$EXEDIR\App\DefaultData\KeePass.ini" "$PROGRAMDIRECTORY"
			ReadEnvStr $0 "PortableApps.comDocuments"
			StrCmp $0 "" ManuallyCheckForDocuments
				IfFileExists "$0\*.*" "" ManuallyCheckForDocuments
				WriteINIStr "$PROGRAMDIRECTORY\KeePass.ini" "KeePass" "KeeLastDir" "$0"
				Goto ReconstructLastDir
				
			ManuallyCheckForDocuments:
				${GetRoot} $EXEDIR $1
				IfFileExists "$1\Documents\*.*" "" SetPathToRoot
				WriteINIStr "$PROGRAMDIRECTORY\KeePass.ini" "KeePass" "KeeLastDir" "$1\Documents"
				Goto ReconstructLastDir
			
			SetPathToRoot:
				WriteINIStr "$PROGRAMDIRECTORY\KeePass.ini" "KeePass" "KeeLastDir" "$1\"
				Goto ReconstructLastDir

	MoveSettings:
		Rename "$SETTINGSDIRECTORY\KeePass.ini" "$PROGRAMDIRECTORY\KeePass.ini"

	ReconstructLastDir:
		ReadINIStr $0 "$PROGRAMDIRECTORY\KeePass.ini" "KeePass" "KeeLastDir"
		${GetRoot} $EXEDIR $1
		${GetRoot} $0 $2
		StrLen $3 $2
		StrCpy $4 $0 "" $3 ;get the last path without the drive letter
		WriteINIStr "$PROGRAMDIRECTORY\KeePass.ini" "KeePass" "KeeLastDir" "$1$4"		
		
		ReadEnvStr $0 "PortableApps.comLocaleWinName"
		StrCmp $0 "" LaunchNow ;if not set, move on
		StrCpy $APPLANGUAGE $0 "" 5 ;just get the language name
		StrCmp $APPLANGUAGE "ENGLISH" "" CheckForEnglishGB
			StrCpy $APPLANGUAGE "Standard"
			Goto GetCurrentLanguage
	CheckForEnglishGB:
		StrCmp $APPLANGUAGE "ENGLISHGB" "" GetCurrentLanguage 
			StrCpy $APPLANGUAGE "Standard"
			Goto GetCurrentLanguage

	GetCurrentLanguage:
		ReadINIStr $0 "$PROGRAMDIRECTORY\KeePass.ini" "KeePass" "KeeLanguage"
		StrCmp $APPLANGUAGE $0 LaunchNow ;if the same, move on
		StrCmp $APPLANGUAGE "Standard" SetAppLanguage ;english is built in, so skip locale file check
		IfFileExists "$PROGRAMDIRECTORY\$APPLANGUAGE.lng" SetAppLanguage LaunchNow

	SetAppLanguage:
		WriteINIStr "$PROGRAMDIRECTORY\KeePass.ini" "KeePass" "KeeLanguage" "$APPLANGUAGE"

	LaunchNow:
		ExecWait $EXECSTRING
		
	CheckRunning:
		Sleep 1000
		FindProcDLL::FindProc "${DEFAULTEXE}"                  
		StrCmp $R0 "1" CheckRunning

	;=== Put the settings file back
	Sleep 500
	Rename "$PROGRAMDIRECTORY\KeePass.ini" "$SETTINGSDIRECTORY\KeePass.ini"
	Goto TheEnd
	
	LaunchAndExit:
		Exec $EXECSTRING

	TheEnd:
		newadvsplash::stop /WAIT
SectionEnd